### UI/Keys: PgUp/Dn, Home, End #############################################
#                                                              vim: ft=dosini
#
# To disable this, use
#
#   echo >>~/.pyroscope/rtorrent.d/.rcignore "bind-navigation-keys.rc.default"
#
# See the "Trouble-Shooting Guide" in the manual for when and why to do that.

pyro.bind_key = navigation_home, 0406, "ui.focus.home="
pyro.bind_key = navigation_end,  0550, "ui.focus.end="
pyro.bind_key = navigation_pgup, 0523, "ui.focus.pgup="
pyro.bind_key = navigation_pgdn, 0522, "ui.focus.pgdn="
