#############################################################################
# Standard PyroScope rTorrent Configuration
#
# To include THIS file into your main "~/.rtorrent.rc", add these commands to it:
#   method.insert = pyro.extended, value|const, 0
#   method.insert = pyro.bin_dir, string|const,
#   import = ~/.pyroscope/rtorrent-pyro.rc.default
#
# And read https://pyrocore.readthedocs.io/en/latest/setup.html#rtorrent-pyro-rc
#
# Remove the ".default" if you want to change something (else your changes
# get over-written on update, when you put them into ``*.default`` files).
# The BETTER way to apply customizations is to use the ``.rcignore`` file
# to disable the specific ``*.rc.default`` file, and then provide your own
# version in an EXTRA file.
#
# Set "pyro.extended" to 1 to activate rTorrent-PS features, but ONLY if
# you actually run that patched version of rTorrent!
#

# Import files from "~/.pyroscope/rtorrent.d" (default versions)
execute.throw = (cat,(pyro.bin_dir),pyroadmin),-q,--create-import,~/.pyroscope/rtorrent.d/*.rc.default
import = ~/.pyroscope/rtorrent.d/.import.rc
