### UI/VIEW: Manually tag (select) items ####################################
#                                                              vim: ft=dosini
#   . = toggle focused item between tagged and untagged
#   T = clear all tags
#   : = show tagged items view

pyro.collapsed_view.add = tagged
view.persistent = tagged

# pyro.view.toggle_visible = ‹viewname› – Toggle visibility of an item for the given view
method.insert = pyro.view.toggle_visible, simple|private,\
    "branch=d.views.has=$argument.0=,view.set_not_visible=$argument.0=,view.set_visible=$argument.0="

# Empty the 'tagged' view, and remove visibility for all items
method.insert = pyro._view_tagged_clear, simple|private,\
    "view.filter=tagged,false= ; d.multicall2 = default, d.views.remove=tagged"

# Key bindings for 'tagged' view management
pyro.bind_key = tagged_toggle, ., "pyro.view.toggle_visible=tagged"
pyro.bind_key = tagged_clear,  T, "pyro._view_tagged_clear="
pyro.bind_key = tagged_view,   :, "ui.current_view.set=tagged"
