# Color scheme for 256 xterm colors - Solarized Blue (http://ethanschoonover.com/solarized)

pyro.color_theme.name    = "Solarized Blue"

ui.color.alarm.set       = "bold 254 on 160"
ui.color.complete.set    = "33"
#ui.color.custom1.set     = 242
ui.color.even.set        = "on 234"
ui.color.focus.set       = "reverse"
ui.color.footer.set      = "bright 254 on 125"
ui.color.incomplete.set  = "136"
ui.color.info.set        = "245"
ui.color.label.set       = "240"
ui.color.leeching.set    = "bold bright 166"
ui.color.odd.set         = ""
ui.color.progress0.set   = "160"
ui.color.progress20.set  = "61"
ui.color.progress40.set  = "64"
ui.color.progress60.set  = "166"
ui.color.progress80.set  = "136"
ui.color.progress100.set = "33"
ui.color.progress120.set = "bold bright 37"
ui.color.queued.set      = "125"
ui.color.seeding.set     = "bold bright 37"
ui.color.stopped.set     = "64"
ui.color.title.set       = "bold bright 230 on 61"
