# Color scheme for 256 xterm colors - Happy Pastel (less harsh default colors with a happy touch)

pyro.color_theme.name    = "Happy Pastel"

ui.color.alarm.set       = "bold 250 on 124"
ui.color.complete.set    = "41"
#ui.color.custom1.set     = 242
ui.color.even.set        = "on 234"
ui.color.focus.set       = "reverse"
ui.color.footer.set      = "bright 123 on 160"
ui.color.incomplete.set  = "221"
ui.color.info.set        = "white"
ui.color.label.set       = "241"
ui.color.leeching.set    = "bold bright 220"
ui.color.odd.set         = "on 232"
ui.color.progress0.set   = "196"
ui.color.progress20.set  = "207"
ui.color.progress40.set  = "202"
ui.color.progress60.set  = "214"
ui.color.progress80.set  = "226"
ui.color.progress100.set = "34"
ui.color.progress120.set = "bold bright 46"
ui.color.queued.set      = "133"
ui.color.seeding.set     = "bold bright 40"
ui.color.stopped.set     = "33"
ui.color.title.set       = "bold bright white on 202"
