### UI/VIEW: Collapse all built-in views ####################################
#                                                              vim: ft=dosini
#
# To disable this, use
#
#   echo >>~/.pyroscope/rtorrent.d/.rcignore "collapse-built-in-views.rc.default"
#

# PS 1.1+ has the built-in views collapsed by default!
branch=(not, (system.has, "collapsed-views")), ((do, \
    "pyro.view.collapsed.toggle = active", \
    "pyro.view.collapsed.toggle = complete", \
    "pyro.view.collapsed.toggle = hashing", \
    "pyro.view.collapsed.toggle = incomplete", \
    "pyro.view.collapsed.toggle = leeching", \
    "pyro.view.collapsed.toggle = main", \
    "pyro.view.collapsed.toggle = name", \
    "pyro.view.collapsed.toggle = seeding", \
    "pyro.view.collapsed.toggle = started", \
    "pyro.view.collapsed.toggle = stopped" ))

# EOF
