# Color scheme for 8 ANSI colors

pyro.color_theme.name    = "Default (8 ANSI colors)"

ui.color.alarm.set       = "white on red"
ui.color.complete.set    = "white"
#ui.color.custom1.set     = "gray"
ui.color.even.set        = ""
ui.color.focus.set       = "reverse"
ui.color.footer.set      = "white on blue"
ui.color.incomplete.set  = "yellow"
ui.color.info.set        = "white"
ui.color.label.set       = "gray"
ui.color.leeching.set    = "yellow"
ui.color.odd.set         = ""
ui.color.progress0.set   = "red"
ui.color.progress20.set  = "red"
ui.color.progress40.set  = "magenta"
ui.color.progress60.set  = "yellow"
ui.color.progress80.set  = "yellow"
ui.color.progress100.set = "green"
ui.color.progress120.set = "green"
ui.color.queued.set      = "magenta"
ui.color.seeding.set     = "green"
ui.color.stopped.set     = "blue"
ui.color.title.set       = "white on blue"
