### UI/VIEW: Uploaded data ##################################################
#                                                              vim: ft=dosini
#   All items, sorted by upload amount.
#   Bound to '°' key (will ONLY work if you use rT-PS)!

pyro.collapsed_view.add = uploaded
pyro.bind_key = uploaded_view, 0260, "view.sort = uploaded ; ui.current_view.set = uploaded"

branch = (and, (system.has, "compare="), (system.has, "d.custom.if_z=")), \
    "view.sort_new = uploaded, \"compare=--+,d.up.total=,pyro._view_sort_timestamp=,d.name=\"", \
    "view.sort_new = uploaded, greater=d.up.total="

branch = (and, (system.has, "compare="), (system.has, "d.custom.if_z=")), \
    "view.sort_current = uploaded, \"compare=--+,d.up.total=,pyro._view_sort_timestamp=,d.name=\"", \
    "view.sort_current = uploaded, greater=d.up.total="
