### TORQUE: View for queue manager job ######################################
#                                                              vim: ft=dosini
#
#   This is not the buggy internal scheduler controlled by scheduler.max_active!

pyro.collapsed_view.add = pyrotorque

view.filter = pyrotorque, ((false))
schedule2 = filter_pyrotorque, 1, 15,\
    "view.filter = pyrotorque,\"or={d.up.rate=,d.down.rate=,d.peers_connected=,not=$d.complete=}\""

pyro.bind_key = pyrotorque_view, Q, "ui.current_view.set=pyrotorque"

# TORQUE: Daemon watchdog
method.insert = pyro.watchdog, simple|private,\
    "execute.nothrow=bash,-c,\"$cat=\\\"test ! -f \\\",$argument.0=,\\\"/run/pyrotorque || \\\",$pyro.bin_dir=,\\\"pyrotorque --cron \\\",$argument.1=, \\\" || true\\\"\""
