### UI/VIEW: Show current messages (bound to '!' in rT-PS) ##################
#                                                              vim: ft=dosini

pyro.collapsed_view.add = messages
view.filter = messages, ((d.message))

branch=pyro.extended=,false=,"view.sort_new = messages,less=d.message="
branch=pyro.extended=,"view.sort_new = messages,\"compare=,d.message=,d.name=\""

branch=pyro.extended=,false=,"view.sort_current = messages,less=d.message="
branch=pyro.extended=,"view.sort_current = messages,\"compare=,d.message=,d.name=\""

method.insert = pyro.ui.messages.show, simple,\
    "ui.current_view.set=messages ;view.sort=messages ;print=$view.size=messages,\" message(s)!\""

pyro.bind_key = messages_view, !, "pyro.ui.messages.show="
